#ifndef __EMITTED_VALIDATE_H
#define __EMITTED_VALIDATE_H

#include <bits/stdc++.h>

typedef void (*feedback_function)(const char*, ...);

const int EXITCODE_AC = 0;
const int EXITCODE_WA = 0;

// declare a constant to be used as user_out.
const std::string FILENAME_USER_OUTPUT = "user_out";
// declare a constant to be used as input.
const std::string FILENAME_INPUT = "input";

#define USAGE "./%s\n"
// we have to define USAGE as dummy because we don't want to modify the lib. too much.

std::ifstream judge_in, judge_ans;
// another ifstream must be used for user_out;
std::ifstream user_out;

char *feedbackdir = NULL;

void vreport_feedback(const char* msg, va_list pvar) {
    assert(stderr);
    vfprintf(stderr, msg, pvar);
}

void report_feedback(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
}

void judge_message(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
}

void judge_error(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
    assert(0);
}

// cleaning up.
void cleanup() {
    fflush(stderr);
    fclose(stderr);
    fflush(stdout);
    fclose(stdout);
    judge_in.close();
    user_out.close();
}

// write another function ``scoring()`` for ac or wa.
void scoring(double score) {
    fprintf(stdout, "%.f", score + 1e-9);
}

void accept() {
    scoring(100.);
    cleanup();
    exit(EXITCODE_AC);
}

void accept_with_score(double scorevalue) {
    scoring(scorevalue);
    cleanup();
    exit(EXITCODE_AC);
}

void wrong_answer(const char* msg, ...) {
    va_list pvar;
    va_start(pvar, msg);
    vreport_feedback(msg, pvar);
    scoring(0.);
    cleanup();
    exit(EXITCODE_WA);
}

void init_io() {
    judge_in.open(FILENAME_INPUT, std::ios_base::in);
    if(judge_in.fail()) {
        judge_error("failed to open input file.\n");
    }
    user_out.open(FILENAME_USER_OUTPUT, std::ios_base::in);
    if(user_out.fail()) {
        judge_error("failed to open user output file.\n");
    }
    judge_ans.open("answer", std::ios_base::in);
    if(judge_ans.fail()) {
        judge_error("failed to open answer file.\n");
    }
}
using namespace std;
void OK(istream &ouf, feedback_function waf) {
    string t;
    if (ouf >> t)
        waf("Sai. Du lieu ra khong dung dinh dang.\n");
    judge_message("OK. Dap an duoc chap nhan!\n");
}

typedef vector<int> vti;
typedef vector<vti> vtv;
vtv dsk;
int n, m;


bool check(int u, int v) {
    if (u < 1 || u > n || v < 1 || v > n)
        return 0;
    for (int x : dsk[u])
        if (x == v)
            return 1;
    return 0;
}

bool check(istream &inf, istream &ans, istream &ouf, feedback_function waf) {
    int u = -1, w = -1, v, as, os;
    ans >> as;
    if (!(ouf >> os))
        waf("Sai. Du lieu ra khong dung dinh dang!\n");
    if (as != os)
        waf("Sai. Gia tri thoi gian gap nhau khong dung!\n");
    if (os != -1) {
        inf >> n >> m;
        dsk = vtv(n + 1, vti(0));
        for (int i = 0; i < m; i++) {
            inf >> u >> v;
            dsk[u].push_back(v);
        }
        for (int i = 0; i <= os; i++) {
            if (!(ouf >> v))
                waf("Sai. Du lieu ra khong dung dinh dang!\n");
            if (i == 0) {
                if (v != 1)
                    waf("Sai. Robot thu nhat khong xuat phat tu dinh 1!\n");
            } else {
                if (!check(u, v))
                    waf("Sai. Cung (%d, %d) khong dung!\n", u, v);
            }
            u = v;
        }

        for (int i = 0; i <= os; i++) {
            if (!(ouf >> v))
                waf("Sai. Du lieu ra khong dung dinh dang!\n");
            if (i == 0) {
                if (v != n)
                    waf("Sai. Robot thu hai khong xuat phat tu dinh %d!\n", n);
            } else {
                if (!check(w, v))
                    waf("Sai. Cung (%d, %d) khong dung!\n", w, v);
            }
            w = v;
        }

        if (u != w)
            waf("Sai. Hai Robot khong gap nhau tai dinh cuoi!\n");
    }
    OK(ouf, waf);
    return 1;
}
#endif

int main(void) {
    init_io();
    check(judge_in, judge_ans, user_out, wrong_answer);
    accept();
}
